package l11_io;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serial;
import java.io.Serializable;

public class ObjectReadTest {

	public static void main(String[] args) {

		final String fileName = "invoice.bin";
		writeFile(fileName);
		readFile(fileName);
	}

	record Gadget(double price, int units, String description) implements Serializable {

		/*
		 The serialization runtime associates with each serializable class a version number,
		 called a serialVersionUID, which is used during deserialization to verify that
		 the sender and receiver of a serialized object have loaded classes for that object
		 that are compatible with respect to serialization. If the receiver has loaded a class
		 for the object that has a different serialVersionUID than that of the corresponding
		 sender's class, then deserialization will result in an InvalidClassException.
		 A serializable class can declare its own serialVersionUID explicitly by declaring
		 a field named "serialVersionUID" that must be static, final, and of type long:
		*/

		@Serial
		private static final long serialVersionUID = 7526471155622776147L;

		private double calculateTotal() {
			return this.price * this.units;
		}

		@Override
		public String toString() {
			return String.format("You've ordered %d units of %s at $%.2f", units, description, price);
		}
	}

	private static void writeFile(String fileName) {
		Gadget[] gadgets = {
				new Gadget(19.99, 12, "Java Mug"),
				new Gadget(9.99, 68, "Duke Juggling Dolls"),
				new Gadget(15.99, 34, "Java T-shirt"),
				new Gadget(3.99, 24, "Java Pin"),
				new Gadget(5.99, 46, "Java Key Chain")
		};

		try (ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(fileName))) {
			out.writeInt(gadgets.length);
			for (Gadget gadget : gadgets) {
				out.writeObject(gadget);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void readFile(String fileName) {
		Gadget gadget;
		double total = 0.0;
		try (ObjectInputStream in = new ObjectInputStream(new FileInputStream(fileName))) {
			int noRecords = in.readInt();
			for (int i = 0; i < noRecords; i++) {
				gadget = (Gadget) in.readObject();
				total += gadget.calculateTotal();
				System.out.println(gadget);
			}
		} catch (IOException | ClassNotFoundException e) {
			e.printStackTrace();
		}
		System.out.println("------------------------------");
		System.out.format("For a TOTAL of: $%.2f\n", total);
	}

}

