package l11_io;

import utilities.ThreadHelperMethods;
import java.io.IOException;
import java.io.PipedReader;
import java.io.PipedWriter;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.ForkJoinPool;

public class PipedIO {

	public static void main(String[] args) {
		try (ExecutorService executor = new ForkJoinPool(2)) {
			Sender sender = new Sender();
			Receiver receiver = new Receiver(sender.getPipedWriter());
			executor.submit(receiver);
			executor.submit(sender);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	static class Sender implements Callable<Void> {
		private final PipedWriter pipedWriter = new PipedWriter();

		public PipedWriter getPipedWriter() {
			return pipedWriter;
		}

		@Override
		public Void call() throws Exception {
			try {
				for (char c = 'A'; c <= 'Z'; c++) {
					pipedWriter.write(c);
					ThreadHelperMethods.safeSleep(800);
				}
				return null;
			} finally {
				System.out.println("closing the writer");
				pipedWriter.close();
			}
		}
	}

	static class Receiver implements Callable<Void> {
		private final PipedReader pipedReader;

		public Receiver(PipedWriter pipedWriter) throws IOException {
			pipedReader = new PipedReader(pipedWriter); // PipedReader ctor throws IOException
		}

		@Override
		public Void call() throws Exception {
			try {
				int data;
				while ((data = pipedReader.read()) != -1) {
					System.out.println("Read: '" + (char)data + "'");
					ThreadHelperMethods.safeSleep(1000);
				}
				return null;
			} finally {
				System.out.println("closing the reader");
				pipedReader.close();
			}
		}
	}
}
