package l11_io;

import utilities.InputValue;
import java.io.IOException;
import java.io.RandomAccessFile;

public class RafObjectTest {

	static class Gadget {
		private double price;
		private final int units;
		private final String description;

		@Override
		public String toString() {
			return String.format("You have %d units of %s at $%.2f",
				units, description, price);
		}

		public Gadget(double price, int units, String description) {
			this.price = price;
			this.units = units;
			this.description = description;
		}

		private void writeRecord(RandomAccessFile out) throws IOException {
			out.writeDouble(price);
			out.writeInt(units);
			out.writeUTF(description);
		}

		private static Gadget readRecord(RandomAccessFile in) throws IOException {
			return new Gadget(in.readDouble(), in.readInt(), in.readUTF());
		}
	}

	public static void main(String[] args) {
		try (RandomAccessFile raf = new RandomAccessFile("invoice.bin", "rw");
			RandomAccessFile positions = new RandomAccessFile("positions.bin", "rw")) {
			int numberOfRecords = writeFile(raf, positions);
			printFile(raf, numberOfRecords);
			updateRecord(raf, getPosition(numberOfRecords, positions));
			printFile(raf, numberOfRecords);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static int writeFile(RandomAccessFile raf, RandomAccessFile positions) throws IOException {

		Gadget[] gadgets = {
				new Gadget(19.99, 12, "Java Mug"),
				new Gadget(9.99, 68, "Duke Juggling Dolls"),
				new Gadget(15.99, 34, "Java T-shirt"),
				new Gadget(3.99, 24, "Java Pin"),
				new Gadget(5.99, 46, "Java Key Chain")
		};

		// write data to file
		raf.setLength(0L);
		positions.setLength(0L);
		for (Gadget gadget : gadgets) {
			positions.writeLong(raf.getFilePointer());
			gadget.writeRecord(raf);
		}
		return gadgets.length;
	}

	private static long getPosition(int numberOfRecords, RandomAccessFile positions) throws IOException {

		int recordNumber;
		do {
			recordNumber = InputValue.getInt("record number (0 - " + (numberOfRecords - 1) + ")");
		} while (recordNumber < 0 || recordNumber >= numberOfRecords);
		positions.seek(recordNumber * 8L);
		return positions.readLong();
	}

	private static void updateRecord(RandomAccessFile raf, long position) throws IOException {

		// read the record
		raf.seek(position);
		Gadget gadget = Gadget.readRecord(raf);

		// get new price
		System.out.println(gadget);
		gadget.price = InputValue.getDouble("new price");

		// update the file
		raf.seek(position);
		gadget.writeRecord(raf);
	}

	private static void printFile(RandomAccessFile raf, int numberOfRecords) throws IOException {

		double total = 0;
		System.out.println();
		raf.seek(0L);
		for (int i = 0; i < numberOfRecords; i++) {
			Gadget gadget = Gadget.readRecord(raf);
			total = total + gadget.units * gadget.price;
			System.out.println(gadget);
		}
		System.out.println("--------------------------------");
		System.out.format("For a TOTAL of: $%.2f\n", total);
		System.out.println();
	}
}

