package l11_io;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

public class RecordIOTest {

	record Gadget(double price, int units, String description) {

		private void writeRecord(DataOutputStream out) throws IOException {
			out.writeDouble(price);
			out.writeInt(units);
			out.writeUTF(description);
		}

		private static Gadget readRecord(DataInputStream in) throws IOException {
			return new Gadget(in.readDouble(), in.readInt(), in.readUTF());
		}

		private double calculateTotal() {
			return this.price * this.units;
		}

		@Override
		public String toString() {
			return String.format("You've ordered %d units of %s at $%.2f",
				units, description, price);
		}
	}

	private static void writeFile(String fileName) {
		Gadget[] gadgets = {
			new Gadget(19.99, 12, "Java Mug"),
			new Gadget( 9.99, 68, "Duke Juggling Dolls"),
			new Gadget(15.99, 34, "Java T-shirt"),
			new Gadget( 3.99, 24, "Java Pin"),
			new Gadget( 5.99, 46, "Java Key Chain")
		};

		try (DataOutputStream out = new DataOutputStream(new FileOutputStream(fileName))) {
			out.writeInt(gadgets.length);
			for (Gadget gadget : gadgets) {
				gadget.writeRecord(out);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void readFile(String fileName) {
		Gadget gadget;
		double total = 0.0;
		try (DataInputStream in = new DataInputStream(new FileInputStream(fileName))) {
			int noRecords = in.readInt();
			for (int i = 0; i < noRecords; i++) {
				gadget = Gadget.readRecord(in);
				total += gadget.calculateTotal();
				System.out.println(gadget);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("------------------------------");
		System.out.format("For a TOTAL of: $%.2f\n", total);
	}

	public static void main(String[] args) {
		final String fileName = "invoice.bin";
		writeFile(fileName);
		readFile(fileName);
	}
}

