package l11_io;

import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class RhymingWords {
	public static void main(String[] args) throws IOException {

		FileReader words = new FileReader("slowa-utf.txt");

		// do the reversing and sorting
		Reader rhymedWords = reverse(sort(reverse(words)));

		// write new list to standard out
		BufferedReader in = new BufferedReader(rhymedWords);
		String input;

		while ((input = in.readLine()) != null)
			System.out.println(input);
		in.close();
	}

	public static Reader reverse(Reader source) throws IOException {

		PipedWriter pipeOut = new PipedWriter();
		new ReverseThread(new PrintWriter(pipeOut), new BufferedReader(source)).start();
		return new PipedReader(pipeOut);
	}

	public static Reader sort(Reader source) throws IOException {

		PipedWriter pipeOut = new PipedWriter();
		new SortThread(new PrintWriter(pipeOut), new BufferedReader(source)).start();
		return new PipedReader(pipeOut);
	}

	static class ReverseThread extends Thread {
		private final PrintWriter out;
		private final BufferedReader in;

		public ReverseThread(PrintWriter out, BufferedReader in) {
			this.out = out;
			this.in = in;
		}

		@Override
		public void run() {
			if (out == null || in == null) {
				return;
			}
			try {
				String input;
				while ((input = in.readLine()) != null) {
					out.println(new StringBuilder(input).reverse());
				}
			} catch (IOException e) {
				System.err.println("ReverseThread run: " + e);
			} finally {
				out.close();
			}
		}
	}

	static class SortThread extends Thread {
		private final PrintWriter out;
		private final BufferedReader in;

		public SortThread(PrintWriter out, BufferedReader in) {
			this.out = out;
			this.in = in;
		}

		public void run() {
			if (out == null || in == null) {
				return;
			}
			try {
				List<String> listOfWords = new ArrayList<>();
				String word;
				while ((word = in.readLine()) != null) {
					listOfWords.add(word);
				}
				Collections.sort(listOfWords);
				for (String string : listOfWords)
					out.println(string);
			} catch (IOException e) {
				System.err.println("SortThread run: " + e);
			} finally {
				out.close();
			}
		}
	}
}

