import java.util.concurrent.*;

//	Recommended way from Oracle API documentation page of ExecutorService

public class ShutDownExecutor {

	public static void shutDownExecutor(ExecutorService executor) {
		shutDownExecutor(executor, Integer.MAX_VALUE);
	}

	public static void shutDownExecutor(ExecutorService executor, int timeLimit) {
		System.out.println("Attempt to shutdown executor");
		//	This will make the executor accept no new threads
		//	and finish all existing threads in the queue
		executor.shutdown();
		try {
			//	Blocks until all tasks have completed execution after a shutdown request,
			//	or the timeout occurs, or the current thread is interrupted, whichever happens first.
			//	Returns true if this executor terminated and false if the timeout elapsed
			//	Throws InterruptedException when interrupted while waiting
			if(!executor.awaitTermination(timeLimit, TimeUnit.SECONDS)) {
				executor.shutdownNow();
				// Wait a while for tasks to respond to being cancelled
				if (!executor.awaitTermination(timeLimit, TimeUnit.SECONDS)) {
					System.err.println("Executor did not terminate");
				}
			}
		} catch (InterruptedException e) {
			System.err.println("Tasks interrupted");
			// (Re-)Cancel if current thread also interrupted
			executor.shutdownNow();
			// Preserve interrupt status
			Thread.currentThread().interrupt();
		} finally {
			System.out.println("Shutdown finished");
			System.out.println();
		}
	}
}

