package l11_io;

import java.io.FileReader;
import java.io.IOException;
import java.io.StreamTokenizer;

public class TokenizerTest {

	public static void main(String[] args) {
		String fileName = (args.length == 0) ? "Copy.java" : args[0];
		try (FileReader reader = new FileReader(fileName)) {
			tokenizerTest(reader);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void tokenizerTest(FileReader reader) throws IOException {

		StreamTokenizer st = new StreamTokenizer(reader);
		st.wordChars('_', '_'); // make the '_' recognizable as part of a word
		st.ordinaryChar('/'); // Don't parse slash as part of numbers
		int lineNum = -1;
		for (int tokenType = st.nextToken(); tokenType != StreamTokenizer.TT_EOF; tokenType = st.nextToken()) {
			int newLineNum = st.lineno();
			if (newLineNum != lineNum) { // new line
				System.out.println();
				System.out.println("[line " + newLineNum + "]");
				lineNum = newLineNum;
			}
			switch (tokenType) {
				case StreamTokenizer.TT_NUMBER -> System.out.format("\t%-12s %.2f%n", "number:", st.nval);
				case StreamTokenizer.TT_WORD -> System.out.format("\t%-12s %s%n", "identifier:", st.sval);
				default -> System.out.format("\t%-12s %c%n", "operator:", (char) st.ttype);
			}
		}
	}
}



