package nio;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.channels.AsynchronousFileChannel;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.nio.file.FileSystems;
import java.nio.file.StandardOpenOption;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

public class AsyncTest {
	public static void main(String[] args) throws IOException, InterruptedException, ExecutionException {
		final InputStream inputStream = new FileInputStream("big.pdf");
		final ReadableByteChannel inputChannel = Channels.newChannel(inputStream);
		final AsynchronousFileChannel outputChannel = AsynchronousFileChannel.open(
				FileSystems.getDefault().getPath("copy.pdf"),
				StandardOpenOption.CREATE, StandardOpenOption.WRITE);
		outputChannel.lock();

		final ByteBuffer buffer = ByteBuffer.allocate(4096*16);
		int position = 0;
		int receivedBytes;
		Future<Integer> lastWrite;

		while ((receivedBytes = inputChannel.read(buffer)) >= 0 || buffer.position() != 0) {
			System.out.println("Received bytes: " + receivedBytes);
			System.out.println("Buffer position: " + buffer.position());
			buffer.flip();
			lastWrite = outputChannel.write(buffer, position);
			// do extra work while asynchronous channel is writing bytes to disk,
			// in perfect case more extra work can be done, not just simple calculations
			position += receivedBytes;
			// extra work is done, we should wait, because we use only one buffer which can be still busy
			int lw = lastWrite.get();
			System.out.println("-----------" + lw);
			buffer.compact();
		}

		outputChannel.close();
		inputChannel.close();
		inputStream.close();
	}
}
