package l11_io.nio;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class FileChannelDemo {

	public static void main(String[] args) {

		final String fileName = "nio_test.txt";
		Path path = Paths.get(fileName);
		// Encodes this String into a sequence of bytes using the platform's default charset,
		// storing the result into a new byte array
		writeFileChannel(path, ByteBuffer.wrap("Hello, nio channels\n".getBytes()));
		readFileChannel(path);
	}

	private static void writeFileChannel(Path path, ByteBuffer byteBuffer) {

		try (FileChannel fileChannel = FileChannel.open(path,
				StandardOpenOption.WRITE, // writable
				StandardOpenOption.CREATE, // create if the file does not exist
				StandardOpenOption.TRUNCATE_EXISTING)) { // remove previous content
			int bytesWritten = fileChannel.write(byteBuffer);
			System.out.println("Channel size:  " + fileChannel.size());
			System.out.println("Bytes written: " + bytesWritten);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void readFileChannel(Path path) {
		try (FileChannel fileChannel = FileChannel.open(path, StandardOpenOption.READ)) {
			ByteBuffer byteBuffer = ByteBuffer.allocate(4); // to show it is working for large files
			while (fileChannel.read(byteBuffer) >= 0) {
				// The limit is set to the current position and then the position is set to zero.
				byteBuffer.flip();
				System.out.print(StandardCharsets.UTF_8.decode(byteBuffer));
//				while(byteBuffer.hasRemaining()) System.out.print((char)byteBuffer.get()); // optionally
				byteBuffer.clear(); // clear the buffer for next read (the position markers actually)
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}

