package l11_io.nio;

import utilities.RandomGenerators;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.StandardCharsets;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Arrays;

public class GatheringAndScattering {

	private static final int numberOfBuffers = 5;
	private static final int lengthBufferSize = 4 * numberOfBuffers; // int for every buffer

	public static void main(String[] args) {

		String[] data = new String[numberOfBuffers];
		for(int i = 0; i < numberOfBuffers; ++i) {
			data[i] = "Contents of data buffer number " + i + " " + "*".repeat(RandomGenerators.getInt(10));
		}
		final String fileName = "sample.txt";
		Path path = Paths.get(fileName);

		// We are going to store data from few sources to the file using GatheringByteChannel
		gathering(data, path);
		scattering(path);
	}

	private static void gathering(String[] data, Path path) {

		// Store the length of data using GatheringByteChannel
		ByteBuffer lengthBuffer = ByteBuffer.allocate(lengthBufferSize);
		int[] len = new int[numberOfBuffers];
		ByteBuffer[] dataBuffer = new ByteBuffer[numberOfBuffers];

		for(int i = 0; i < numberOfBuffers; ++i) {
			len[i] = data[i].length(); // get the lengths
			dataBuffer[i] = ByteBuffer.wrap(data[i].getBytes()); // prepare the buffers
		}

		lengthBuffer.asIntBuffer().put(len); // put lengths into buffer
		System.out.println("Gathering: length = " + Arrays.toString(len));

		// Write data to the file
		try(FileChannel fileChannel = FileChannel.open(path,
				StandardOpenOption.CREATE, // create if does not exist
				StandardOpenOption.WRITE, // writable
				StandardOpenOption.TRUNCATE_EXISTING)) { // remove previous content
			fileChannel.write(lengthBuffer); // write lengths
			fileChannel.write(dataBuffer); // write data
			lengthBuffer.clear();
			for (int i = 0; i < numberOfBuffers; i++) {
				dataBuffer[i].clear();
			}
		} catch(IOException e) {
			e.printStackTrace();
		}
	}

	private static void scattering(Path path) {

		ByteBuffer lengthBuffer = ByteBuffer.allocate(lengthBufferSize);
		ByteBuffer[] dataBuffer = new ByteBuffer[numberOfBuffers];

		try(FileChannel fileChannel = FileChannel.open(path, StandardOpenOption.READ)) {

			int[] len = new int[numberOfBuffers];
			// Read lengths first to get the length of data
			fileChannel.read(lengthBuffer);
			lengthBuffer.flip();
			lengthBuffer.asIntBuffer().get(len);
			lengthBuffer.clear();
			System.out.println("Scattering: length = " + Arrays.toString(len));

			// Allocate data buffers
			for(int i = 0; i < numberOfBuffers; ++i) {
				dataBuffer[i] = ByteBuffer.allocate(len[i]);
			}

			// Read data from the channel
			System.out.println();
			fileChannel.read(dataBuffer); // read to array of buffers
			for(int i = 0; i < numberOfBuffers; ++i) {
				dataBuffer[i].flip();
				System.out.println(StandardCharsets.UTF_8.decode(dataBuffer[i]));
				dataBuffer[i].clear();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
