package nio;

// The Java NIO Files class (java.nio.file.Files) provides several
// methods for manipulating files in the file system.

import java.io.IOException;
import java.nio.file.*;

public class NIOFile {
	public static void main(String[] args) {
		fileMethods();
	}

	private static void fileMethods() {
		// file exists?
		Path path = Paths.get("src/nio/NIOFile.java");
		System.out.println("File " + path + " exists: " + Files.exists(path));

		// create directory
		Path pathNew = Paths.get("subdir");
		try {
			Path newDir = Files.createDirectory(pathNew);
			System.out.println("File " + newDir + " exists: " + Files.exists(newDir));
		} catch (FileAlreadyExistsException e) {
			// the directory already exists
			System.out.println("The directory " + pathNew + " already exists");
		} catch (IOException e) {
			// something else went wrong
			e.printStackTrace();
		}

		// copy file
		Path sourcePath = Paths.get("Copy.java");
		Path destinationPath = Paths.get("CopyCopy.java");
		try {
//			Files.copy(sourcePath, destinationPath);
			Files.copy(sourcePath, destinationPath, StandardCopyOption.REPLACE_EXISTING);
			System.out.println("File " + destinationPath + " exists: " + Files.exists(destinationPath));
		} catch (FileAlreadyExistsException e) {
			System.out.println("destination file already exists");
		} catch (IOException e) {
			// something else went wrong
			e.printStackTrace();
		}

		// move file
		sourcePath = Paths.get("CopyCopy.java");
		destinationPath = Paths.get("subdir", "CopyMoved.java");
		try {
//			Files.move(sourcePath, destinationPath);
			Files.move(sourcePath, destinationPath, StandardCopyOption.REPLACE_EXISTING);
			System.out.println("File " + destinationPath + " exists: " + Files.exists(destinationPath));
		} catch (IOException e) {
			//moving file failed.
			e.printStackTrace();
		}

		// delete file
		path = Paths.get("subdir/CopyMoved.java");
		try {
			Files.delete(path);
			System.out.println("File " + path + " exists: " + Files.exists(path));
		} catch (IOException e) {
			//deleting file failed
			e.printStackTrace();
		}
	}
}
