package nio;

// The Java NIO Files class (java.nio.file.Files) provides several
// methods for manipulating files in the file system.

import java.io.File;
import java.io.IOException;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;

public class NIOFileFull {
	public static void main(String[] args) {
		fileMethods();
	}

	private static void fileMethods() {
		// file exists?
		Path path = Paths.get("src/nio/NIOFile.java");
		System.out.println("File " + path + " exists: " + Files.exists(path));

		// create directory
		Path pathNew = Paths.get("subdir");
		try {
			Path newDir = Files.createDirectory(pathNew);
			System.out.println("File " + newDir + " exists: " + Files.exists(newDir));
		} catch (FileAlreadyExistsException e) {
			// the directory already exists
			System.out.println("The directory " + pathNew + " already exists");
		} catch (IOException e) {
			// something else went wrong
			e.printStackTrace();
		}

		// copy file
		Path sourcePath = Paths.get("Copy.java");
		Path destinationPath = Paths.get("CopyCopy.java");
		try {
//			Files.copy(sourcePath, destinationPath);
			Files.copy(sourcePath, destinationPath, StandardCopyOption.REPLACE_EXISTING);
			System.out.println("File " + destinationPath + " exists: " + Files.exists(destinationPath));
		} catch (FileAlreadyExistsException e) {
			System.out.println("destination file already exists");
		} catch (IOException e) {
			// something else went wrong
			e.printStackTrace();
		}

		// move file
		sourcePath = Paths.get("CopyCopy.java");
		destinationPath = Paths.get("subdir", "CopyMoved.java");
		try {
//			Files.move(sourcePath, destinationPath);
			Files.move(sourcePath, destinationPath, StandardCopyOption.REPLACE_EXISTING);
			System.out.println("File " + destinationPath + " exists: " + Files.exists(destinationPath));
		} catch (IOException e) {
			//moving file failed.
			e.printStackTrace();
		}

		// delete file
		path = Paths.get("subdir/CopyMoved.java");
		try {
			Files.delete(path);
			System.out.println("File " + path + " exists: " + Files.exists(path));
		} catch (IOException e) {
			//deleting file failed
			e.printStackTrace();
		}

		// walk file tree
		Path startPath = Paths.get("src");
		System.out.println();
		try {
			Files.walkFileTree(startPath, new FileVisitor<>() {
				@Override
				public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) {
					System.out.println("pre visit dir:" + dir);
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) {
					System.out.println("visit file: " + file);
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult visitFileFailed(Path file, IOException exc) {
					System.out.println("visit file failed: " + file);
					return FileVisitResult.CONTINUE;
				}

				@Override
				public FileVisitResult postVisitDirectory(Path dir, IOException exc) {
					System.out.println("post visit directory: " + dir);
					return FileVisitResult.CONTINUE;
				}
			});
		} catch (IOException e) {
			e.printStackTrace();
		}

		// search for a file
		System.out.println();
		Path rootPath = Paths.get("src");
		String fileToFind = File.separator + "Copy.java";

		try {
			Files.walkFileTree(rootPath, new SimpleFileVisitor<>() {
				@Override
				public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
					String fileString = file.toAbsolutePath().toString();
//					System.out.println("pathString = " + fileString);
					if (fileString.endsWith(fileToFind)) {
						System.out.println("file found at path: " + file.toAbsolutePath());
						return FileVisitResult.TERMINATE;
					}
					return FileVisitResult.CONTINUE;
				}
			});
		} catch(IOException e){
			e.printStackTrace();
		}
	}
}
