package l11_io.nio;

import utilities.ThreadHelperMethods;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.Pipe;
import java.nio.charset.StandardCharsets;

public class NIOPipeExample {

	private static Pipe pipe;

	public static void main(String[] args) {
		try {
			pipe = Pipe.open(); // The Pipe is created
		} catch (IOException e) {
			e.printStackTrace();
		}
		new Thread(NIOPipeExample::sender).start();
		new Thread(NIOPipeExample::receiver).start();
	}

	private static void sender() {

		try (Pipe.SinkChannel sinkChannel = pipe.sink()) {
			ByteBuffer byteBuffer = ByteBuffer.allocate(8);
			byteBuffer.clear();
			for(char c = 'A'; c <= 'Z'; ++c) {
				byteBuffer.put((c + "\n").getBytes());
				// Calling flip() sets the position back to 0, and sets the limit to where position just was.
				byteBuffer.flip();
				sinkChannel.write(byteBuffer);
				// make the Buffer ready for writing again.
				byteBuffer.clear();
				ThreadHelperMethods.safeSleep(500);
			}
			System.out.println("Sender done ...");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private static void receiver() {

		try(Pipe.SourceChannel sourceChannel = pipe.source()) {
			ByteBuffer byteBuffer = ByteBuffer.allocate(16);
			while (sourceChannel.read(byteBuffer) >= 0) {
				byteBuffer.flip();
				System.out.print(StandardCharsets.UTF_8.decode(byteBuffer));
				byteBuffer.clear();
				ThreadHelperMethods.safeSleep(2000);
			}
			System.out.println("Receiver done ...");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
